use crate::admin::domain::AdminRepository;
use crate::admin::domain::*;
use crate::admin::state::AdminRegistry;
use async_trait::async_trait;
use entity;
use log::{debug, warn};
use sea_orm::{Database, DatabaseConnection, EntityTrait};
use serde_json::{json, Value};

struct UserRepository {
    connection: DatabaseConnection,
}

impl UserRepository {
    pub fn new(connection: DatabaseConnection) -> Self {
        UserRepository {
            connection: connection,
        }
    }
}

#[async_trait]
impl AdminRepository for UserRepository {
    async fn info(&self, _: &RepositoryContext) -> RepositoryInfo {
        RepoInfo {
            name: "User",
            lookup_key: "id",
            display_list: &["id"],
            fields: &[],
        }
        .into()
    }

    async fn get(&self, model: &RepositoryContext, id: LookupKey) -> Option<RepositoryItem> {
        None
    }

    async fn list(&self, model: &RepositoryContext) -> RepositoryList {
        let results = entity::User::find()
            .all(&self.connection)
            .await
            .expect("Error loading users.");
        let repository_items: Vec<RepositoryItem> = results
            .into_iter()
            .filter_map(|item| {
                match serde_json::to_value(item) {
                    Ok(fields) => Some(RepositoryItem {
                        fields,
                        detail_url: None, // replace with actual value if available
                        change_url: None, // replace with actual value if available
                    }),
                    Err(_) => None,
                }
            })
            .collect();
        RepositoryList::List {
            values: repository_items,
        }
    }

    async fn create(
        &mut self,
        model: &RepositoryContext,
        mut data: Value,
    ) -> Option<RepositoryItem> {
        None
    }

    async fn update(
        &mut self,
        model: &RepositoryContext,
        id: LookupKey,
        data: Value,
    ) -> Option<RepositoryItem> {
        None
    }

    async fn replace(
        &mut self,
        model: &RepositoryContext,
        id: LookupKey,
        data: Value,
    ) -> Option<RepositoryItem> {
        None
    }

    async fn delete(&mut self, _: &RepositoryContext, id: LookupKey) -> Option<Value> {
        None
    }
}
